/**
 * filename : Dictionary.java
 * package  : de.renem.android.dictionary
 *
 * Dictionary ruft 3 verschiedene Activitys auf. 
 * 
 * 1. DictionaryEdit (ndert einen Datensatz)
 * 2. DictionaryAdd (Datensatz hinzufgen)
 * 3. DictionaryList (Datenstze auflisten)
 *
 * DictionaryEdit und DictionaryAdd werden zum Verstndniss auch aus DictionaryList
 * ber ein Kontextmen aufgerufen. 
 * 
 * Wegen der bersicht und der Flexibilitt, wird die SQL-Datenbank nicht einmal 
 * in der Hauptaktivitt (Dictionary) geffnet und anschlieend an anderen Aktivitten
 * (DictionaryList, DictionaryEdit) weiter gereicht. Lediglich die Satznummer (_id) wird
 * bergeben. Wenn also aus der Liste heraus ein Datensatz gendert werden soll, startet
 * z.B. aus DictionaryList die Aktivitt DictionaryEdit. DictionaryEdit ffnet die Datenbank 
 * mit der Abfrage auf die gewnschte _id. Ein Test mit 10.000 Datenstze auf einem HTC-Magic 
 * verlief erfolgreich und ohne Verzgerung. Lediglich das ffnen der ListView dauert
 * so ca. 1 Sekunde. Dies liegt daran das 10.000 Datenstze in ein Array (ber den Adapter)
 * gepackt werden.
 * 
 * DictionaryEdit wird zum bearbeiten und zur Neuanlage verwendet. bergibt man dem Activity 
 * (ber das Bndel) die _id mit dem Wert "0", bedeutet dies, Datensatz neu anlegen.
 *  
 * SeekEdit gibt lediglich einen Wert zurck. Dieser wird fr die neue SQL-Abfrage verwendet.
 * Im Grunde ist es kein suchen sondern filtern. Es werden die Eintrge in der Liste angezeigt 
 * die das Suchwort von seekEdit enthalten. Die Abfrage wird mit like durchgefhrt.  
 *
 * Using JRE 1.6.0_21
 *
 * @author renem
 * @date 14.10.2010
 * @version 1.0
 *
 * weitere Beispiele unter www.android.renem.de
 * 
 */
package de.renem.android.dictionary;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.widget.Button;
import android.widget.TextView;


public class Dictionary extends Activity {

	// Mit dieser Datenbank arbeiten alle Activitys.
	public static final String SQL_DATABASE = "renem_dictionary.db";

	// Fr die ResultActivity
	private static final int REQUEST_DICTIONARYEDIT = 1;
    private static final int REQUEST_DICTIONARYLIST = 2;
    private static final int REQUEST_DICTIONARYADD = 3;

    // Kleine Info was von den Activitys zurck gegeben wird.
    private TextView mInfo; 

	/* (non-Javadoc)
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
        Log.e("MyApp", "-- ON CREATE --");

		setContentView(R.layout.main);
		
        Button btnEdit = (Button) findViewById(R.id.Edit);
		btnEdit.setOnTouchListener(onTouchButton);

        Button btnAdd = (Button) findViewById(R.id.Add);
		btnAdd.setOnTouchListener(onTouchButton);

        Button btnList = (Button) findViewById(R.id.List);
		btnList.setOnTouchListener(onTouchButton);

		TextView myMessage = (TextView) findViewById(R.id.message);
        myMessage.setText("Beispiel Wrterbuch");    
	
		mInfo = (TextView) findViewById(R.id.Info);
        mInfo.setText("");    

	}	

	OnTouchListener onTouchButton = new OnTouchListener() {
		
		public boolean onTouch(View v, MotionEvent event) {

	    	switch (v.getId()) {
	    		case R.id.Edit:
	    			if( event.getAction() == MotionEvent.ACTION_UP){
	    				Intent intent = new Intent(getApplicationContext(), DictionaryEdit.class);
		    			Bundle bundle = new Bundle();
		    			// 1 = Datensatz mit dieser _id wird gendert. 
		    			bundle.putLong("_id", 1);
		    			intent.putExtra("bundle", bundle);
	    				startActivityForResult(intent, REQUEST_DICTIONARYEDIT);
	    			}
	    			break;

	    		case R.id.Add:
	    			if( event.getAction() == MotionEvent.ACTION_UP){
		    			Intent intent = new Intent(getApplicationContext(), DictionaryEdit.class);
		    			Bundle bundle = new Bundle();
		    			// 0 = Neuanlage
		    			bundle.putLong("_id", 0);
		    			intent.putExtra("bundle", bundle);
		    			startActivityForResult(intent,REQUEST_DICTIONARYADD);
	    			}
	    			break;
	    		
	    		case R.id.List:
	    			if( event.getAction() == MotionEvent.ACTION_UP){
	    				Intent intent = new Intent(getApplicationContext(), DictionaryList.class);
	    				startActivityForResult(intent, REQUEST_DICTIONARYLIST);
	    			}
	    			break;
	    	}
	        return false;
		}
		
	};
	
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        // Rckgabe der Activitys. Das Bndel wird in den jeweiligen Activitys
    	// (DictionaryEdit, DictionaryAdd) mit Informationen gefllt.
        switch (requestCode) {
        case REQUEST_DICTIONARYEDIT:
        	if (resultCode == Activity.RESULT_OK) {
            	Bundle bundle = data.getExtras().getBundle("bundle");
				mInfo.setText(bundle.getString("Result"));    
            }
            if (resultCode == Activity.RESULT_CANCELED) {
				mInfo.setText("Abbruch Edit!");    
            }
            break;
        case REQUEST_DICTIONARYADD:
            if (resultCode == Activity.RESULT_OK) {
            	Bundle bundle = data.getExtras().getBundle("bundle");
				mInfo.setText(bundle.getString("Result"));    
            }
            if (resultCode == Activity.RESULT_CANCELED) {
				mInfo.setText("Abbruch Add!");    
            }
            break;

        case REQUEST_DICTIONARYLIST:
			mInfo.setText("Die ListView wurde aufgerufen");    
            break;

        }
    }
    
}