/**
 * filename : DictionaryEdit.java
 * package  : de.renem.android.dictionary
 *
 * Stellt einen Bearbeiten/Neuanlagen-Dialog zur Verfgung.
 * Bei bergabe _id = 0 wird eine Neuanlage durchgefhrt. 
 * Ist die _id grer 0 wird ein Datensatz gendert. 
 *
 * Using JRE 1.6.0_21
 *
 * @author renem
 * @date 14.10.2010
 * @version 1.0
 *
 * weitere Beispiele unter www.android.renem.de
 * 
 */
package de.renem.android.dictionary;


import android.app.Activity;
import android.content.ContentValues;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;
import android.widget.Button;
import android.widget.EditText;

public class DictionaryEdit extends Activity {

    private RMDatabase mDB = null; 
	
	private Bundle mBundle = new Bundle();
    private EditText mSource, mTarget; 
	
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Log.e("MyApp", "+-- ON CREATE --+ DictionaryEdit");

        Bundle extras = getIntent().getExtras();

		// Objekt der RMDatabase erzeugen
		mDB = new RMDatabase();

		// Wenn nicht erfolgreich geffnet dann raus 
		if(!mDB.open(Dictionary.SQL_DATABASE)){
			finish();
			return;
		}
		
        setContentView(R.layout.dictionary_edit);

    	if (extras != null) {
    		mBundle = extras.getBundle("bundle");
    	}

    	Button pshCancel = (Button) findViewById(R.id.pshCancel);
		pshCancel.setOnTouchListener(onTouchButton);
		
        Button pshOK = (Button) findViewById(R.id.pshOK);
		pshOK.setOnTouchListener(onTouchButton);

    	mSource = (EditText) findViewById(R.id.source);
		mTarget = (EditText) findViewById(R.id.target);

		// Entweder einen Datensatz bearbeiten oder fr die Neuanalge alle
		// Datenstze verwenden.
		String sql;
		if(mBundle.getLong("_id")>0){
			sql = "SELECT * FROM DICTIONARY WHERE _id = "+mBundle.getLong("_id");
		}else{
			sql = "SELECT * FROM DICTIONARY";
		}
		Cursor cur = mDB.getDatabase().rawQuery (sql, null);
		
		if (cur != null) {
			// Move ist wichtig, damit der Zeiger nicht ber einen Datensatz hinaus geht.
			cur.moveToFirst();
			// Datensatz wird gendert. Bei 0 wre es eine Neuanlage
			if(mBundle.getLong("_id")>0){
				// sicher ist sicher..
				if(cur.getCount() > 0){
					mSource.setText(cur.getString(cur.getColumnIndexOrThrow("Source")));    
					mTarget.setText(cur.getString(cur.getColumnIndexOrThrow("Target")));    
				}
			}
		}

    }

OnTouchListener onTouchButton = new OnTouchListener() {
		
		@Override
		public boolean onTouch(View v, MotionEvent event) {

	    	switch (v.getId()) {
	    		case R.id.pshCancel:
	    			if( event.getAction() == MotionEvent.ACTION_UP){
	    	            Intent intent = new Intent();
	    	            // Nichts passiert..
	    	            setResult(Activity.RESULT_CANCELED, intent);
	    	            finish();
	    	            break;
	    				}
	    			break;
	    		case R.id.pshOK:
	    			if( event.getAction() == MotionEvent.ACTION_UP){
	    	            Intent intent = new Intent();
	    	            if(mBundle.getLong("_id") <= 0){
	    	            	// Bei 0 ist es eine Neuanlage. 
	    	            	// Eine Information fr die Rckgabe gibt es auch....
	    	            	if( addRec()){
			    	            mBundle.putString("Result","Neuanlage erfolgreich aus DictionaryEdit!!!");
		    	            }else{
			    	            mBundle.putString("Result","Neuanlage Fehler!!! aus DictionaryEdit!!!");
		    	            }
	    				}else{
	    	            	// Grer 0 wird der aktuelle Satz bearbeitet. 
	    	            	// Eine Information fr die Rckgabe gibt es auch....
		    	            if( updateRec()){
			    	            mBundle.putString("Result","nderung erfolgreich aus DictionaryEdit!!!");
		    	            }else{
			    	            mBundle.putString("Result","nderung Fehler!!! aus DictionaryEdit!!!");
		    	            }
	    				}
	    	            intent.putExtra("bundle", mBundle);
	    	            setResult(Activity.RESULT_OK, intent);
	    	            finish();
	    	            break;
	    			}
	    			break;
	    	}
	        return false;
		}
	};

	private boolean addRec(){
		return mDB.getDatabase().insert("DICTIONARY", null, getContentValues()) > 0;
	}
	
	private boolean updateRec(){
		// Welcher Datensatz gendert werden soll, steht im Bndel. Es wurde der
		// Activity bergeben.
		return mDB.getDatabase().update("DICTIONARY", getContentValues(),
				"_id = "+mBundle.getLong("_id"), null) > 0;
	}

	private ContentValues getContentValues() {
		
		// Fr die Neuanlage oder das ndern eines Datensatzes werden
		// die selben Feld-Attribute verwendet.
		ContentValues values = new ContentValues();

		values.put("Source", mSource.getText().toString());
		values.put("Target", mTarget.getText().toString());

		return values;
	}
	
	public void onStart() {
        super.onStart();
        Log.e("MyApp", "-- ON START -- DictionaryEdit");
    }

    public void onResume() {
        super.onResume();
       Log.e("MyApp", "-- ON RESUME -- DictionaryEdit");
    }

    public void onPause() {
        super.onPause();
        Log.e("MyApp", "-- ON PAUSE -- DictionaryEdit");
    }
    
    public void onStop() {
        super.onStop();
        Log.e("MyApp", "-- ON STOP -- DictionaryEdit");
    }

    public void onDestroy() {
        super.onDestroy();
        Log.e("MyApp", "-- ON DESTROY -- DictionaryEdit");
        // Datenbank schlieen.
		mDB.close();
    }
    	
}
