/**
 * filename : DictionaryList.java
 * package  : de.renem.android.dictionary
 *
 * Listet Datenstze aus der Tabelle DICTIONARY. Mit Hilfe eines
 * Kontextmens (LongClick auf ListView) sind nderungen mglich.
 *
 * Using JRE 1.6.0_21
 *
 * @author renem
 * @date 14.10.2010
 * @version 1.0
 *
 * weitere Beispiele unter www.android.renem.de
 * 
 */
package de.renem.android.dictionary;


import android.app.Activity;
import android.app.ListActivity;
import android.content.Intent;
import android.database.Cursor;
import android.os.Bundle;
import android.util.Log;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemLongClickListener;
import android.widget.ListView;
import android.widget.SimpleCursorAdapter;
import android.widget.Toast;

public class DictionaryList extends ListActivity {

    private RMDatabase mDB = null; 
    
    // Bei einem LongClick sollte man sich die aktuelle Satznummer 
    // (_id) merken.
    private long mCurrentRecID = 0;
    
    // Adapter fr die ListView
    private SimpleCursorAdapter mAdapter;

	// Fr die Rckgabe der Aktivitten
    private static final int REQUEST_DICTIONARYEDIT = 1;
    private static final int REQUEST_DICTIONARYADD = 2;
    private static final int REQUEST_DICTIONARYSEEK = 3;

    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Log.e("MyApp", "+-- ON CREATE --+ DictionaryList");
		
        // Objekt der RMDatabase erzeugen
		mDB = new RMDatabase();

		// Datenbank ffnen. Geht was schief, dann raus!
		if(!mDB.open(Dictionary.SQL_DATABASE)){
			finish();
			return;
		}
		
		// Den Inhalt der ListView erzeugen. Angezeigt dir das Attribut Auswahl.
		// Hier werden mehrere Attribute mit Pipes angehangen.
		// Es ginge auch : "SELECT Source AS Auswahl FROM DICTIONARY"
        initListView("SELECT _id,  Source || ' / ' || Target || ' _id = ' || _id AS Auswahl FROM DICTIONARY");
        
        // Fr den LongClick. Das Kontextmen (ebenfalls mit LongClick) gibt keine _id 
        // der Listview her. Dies scheint eine Elegante Lsung zu sein.
        ListView listView = getListView();
        
        // Beim LongClick im Kontextmen bekomme ich keine ID des gewhlten Eintrages.
        listView.setOnItemLongClickListener(new OnItemLongClickListener() {
       	  @Override 
      	  public boolean onItemLongClick(AdapterView<?> parent, View v, int position, long id) {
            // Aktuelle _id merken.
       		mCurrentRecID = id;
    		return false;
        	  }
       	 });
        
		// Ein Kontextmen fr die ListView 
        registerForContextMenu(getListView());
        
    }

	public void onCreateContextMenu(ContextMenu menu, View v,
			ContextMenuInfo menuInfo) {
		super.onCreateContextMenu(menu, v, menuInfo);
		// Men siehe res/menu
		menu.setHeaderTitle("Titel");
		MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.menu_dictionary_list, menu);
	}

	public boolean onContextItemSelected(MenuItem item) {
		switch (item.getItemId()) {
        case R.id.men_edit:
        	// Datensatz editieren. Daher wird die _id des Datensatzes mitgegeben 
        	Intent intent = new Intent(getApplicationContext(), DictionaryEdit.class);
			Bundle bundle = new Bundle();
			bundle.putLong("_id", mCurrentRecID);
			intent.putExtra("bundle", bundle);
			startActivityForResult(intent,REQUEST_DICTIONARYEDIT);
        	return true;
        case R.id.men_add:
        	// Neuanlage. Daher wird die _id mit 0 bergeben.
        	Intent intent2 = new Intent(getApplicationContext(), DictionaryEdit.class);
			Bundle bundle2 = new Bundle();
			bundle2.putLong("_id", 0);
			intent2.putExtra("bundle", bundle2);
			startActivityForResult(intent2,REQUEST_DICTIONARYADD);
        	return true;
        case R.id.men_delete:
        	// Aktueller Datensatz lschen. 
        	if(mDB.getDatabase().delete("DICTIONARY", "_ID = "+mCurrentRecID, null) > 0) {
        		Toast.makeText(getApplicationContext(), "lschen erfolgreich", Toast.LENGTH_LONG).show();
        	}else{
        		Toast.makeText(getApplicationContext(), "Fehler beim lschen", Toast.LENGTH_LONG).show();
        	}
        	// Den Cursor aktualisieren.
        	mAdapter.getCursor().requery();
        	return true;
        case R.id.men_seek:
			// Einen filter erstellen. Die neue Abfrage wird in der Result-Methode erzeugt. 
        	Intent intent3 = new Intent(getApplicationContext(), SeekEdit.class);
			startActivityForResult(intent3,REQUEST_DICTIONARYSEEK);
        	return true;
		}
		return super.onContextItemSelected(item);
	}
    
    public void onStart() {
        super.onStart();
        Log.e("MyApp", "-- ON START -- DictionaryList");
    }

    public void onResume() {
        super.onResume();
       Log.e("MyApp", "-- ON RESUME -- DictionaryList");
    }

    public void onPause() {
        super.onPause();
        Log.e("MyApp", "-- ON PAUSE -- DictionaryList");
    }
    
    public void onStop() {
        super.onStop();
        Log.e("MyApp", "-- ON STOP -- DictionaryList");
    }

    public void onDestroy() {
        super.onDestroy();
        Log.e("MyApp", "-- ON DESTROY -- DictionaryList");
        // Datenbank schlieen.
		mDB.close();
    }
    
    @Override
    protected void onListItemClick(ListView l, View v, int position, long id) {

    	// Einfacher Klick auf ein Item der ListView
    	String action = getIntent().getAction();
        if (Intent.ACTION_PICK.equals(action) || Intent.ACTION_GET_CONTENT.equals(action)) {
        	//Log.d("MyApp","aaaaaaaaaa pos "+position+" id = "+id);
        	//setResult(RESULT_CANCELED, new Intent().setData(uri));
        } else {

        	Toast.makeText(getApplicationContext(), "Auswahl id : "+id, Toast.LENGTH_LONG).show();
        }
    }

    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        switch (requestCode) {
        case REQUEST_DICTIONARYEDIT:
            if (resultCode == Activity.RESULT_OK) {
            	mAdapter.getCursor().requery();
            }
            break;
        case REQUEST_DICTIONARYADD:
            if (resultCode == Activity.RESULT_OK) {
            	// Cursor aktualisieren. Die Neuanlage soll ja sofort
            	// in der ListView sichtbar werden. 
            	mAdapter.getCursor().requery();
            }
            break;
        case REQUEST_DICTIONARYSEEK:
        	if (resultCode == Activity.RESULT_OK) {
            	// Im Bndel steckt der Such bzw. Filterbegriff von SeekEdit.
        		Bundle bundle = data.getExtras().getBundle("bundle");
            	if(bundle != null){
            		// Neu Abfrage erstellen und der ListView zuweisen.
            		String sql = "SELECT _id,  Source || ' / ' || Target || ' _id = ' || _id AS Auswahl FROM DICTIONARY WHERE Auswahl like '%"+bundle.getString("Result")+"%'";
            		initListView(sql);
            	}
            }
            break;
        }
    }

    private void initListView(String sql){
    	
    	// Abfrage erstellen. In cur ist das Ergebnis der Abfrage.
        Cursor cur = mDB.getDatabase().rawQuery(sql, null);
        
        if (cur != null) {
        	// Einen Adapter ertellen und der ListView zuweisen..
        	startManagingCursor(cur);
            mAdapter = new SimpleCursorAdapter(this, R.layout.dictionary_list, cur,
                    new String[] { "Auswahl" }, new int[] { android.R.id.text1 });
            setListAdapter(mAdapter);
        }

    }
    
}
