/*

 *  So wirds gemacht :-)
 * 
 *	// Objekt der RMDatabase erzeugen
 *	RMDatabase mDB = new RMDatabase();
 *
 *	// Datenbank ffnen
 *	mDB.openDatabaseOnSDCard("renem_sql_sdcard_more.db");
 *
 *	// Wenn sich die Versionsnummer ndert, werden 
 *	// Tabellen erzeugt und Strukturen angepasst.
 *  // Dazu dient die validateDatabase().
 *	// 0 = Es wird immer angepasst (nur zu Testzwecken)...
 *	mDB.setVersion(0);
 *
 *	// Wenn nicht erfolgreich geffnet dann raus 
 *	if(!mDB.isOpen()){
 *		finish();
 *		return;
 *	}
 *
 *	// Bei einer neuen Version wird hier die Datenbank angepasst.
 *	mDB.validateDatabase();
 *
 *  // Einen Datensatz hinzufgen
 *	ContentValues values = new ContentValues();
 *	values.put("MyTextUnique", "c");
 *	values.put("MyTextNotNull", "c");
 *	values.put("New_Unique1", "c");
 *	values.put("New_Unique2", "c");
 *	values.put("MyNumeric", 123456789);
 *	values.put("MyReal", 366.98);
 *	mDB.getDatabase().insert("Tabelle1", null, values);
 *
 *	// Datenbank schlieen. Am besten in der onDestroy().
 *	mDB.close();
 *
 * -------------- Oder noch einfacher Anfang. --------------
 * 
 *	// Objekt der RMDatabase erzeugen
 *	RMDatabase mDB = new RMDatabase();
 *
 *	// Wenn nicht erfolgreich geffnet dann raus 
 *	if(!mDB.open("renem_sql_sdcard_more.db")){
 *		finish();
 *		return;
 *	}
 *
 *  // Datenbank schlieen. Am besten in der onDestroy().
 *	mDB.close();
 *
 * -------------- Oder noch einfacher Ende. --------------
 *
 *  Fertig! Hilfreich sind Transaktionen...
 *
 *  /////////////////////////////////////////////////////////////////
 *  Der Vorteil einer Transaktion ist, dass sie entweder vollstndig
 *  ausgefhrt wird oder keine Auswirkung hat. Hier ein Beispiel wie
 *  man diese in Android benutzen kann. Auerhalb dieser Klasse 
 *  natrlich nur mit mDB.getDatabase().beginTransaction() usw.  
 *  /////////////////////////////////////////////////////////////////
 *
 *	mDatabase.beginTransaction();
 *
 *	try{
 *
 *    // Datenbanktransaktionen hier durchfhren.
 *    // Nur wenn alles ohne Fehler durchlaufen ist, 
 *    // schreibt Transaction die Daten in die Datenbank.
 *    // Bei einem Fehler werden die Daten in endTransaction
 *    // zurck gesetzt (RollBack).    
 *
 *	  mDatabase.setTransactionSuccessful();
 *
 *    // Mache keine Anweiungen zwischen
 *    // setTransactionSuccessful und endTransaction
 *
 *	}catch(Exception e){
 * 
 *	  //Ende der Transaktion bei einem geworfenen Fehler
 *
 *    mDatabase.endTransaction();
 *
 *    throw e;
 *
 *   }
 *
 *   //Ende der Transaktion bei keinem Fehler
 *
 *   mDatabase.endTransaction();
 *
 *  }
 * 
 */

package de.renem.android.dictionary;

import android.content.ContentValues;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteException;
import android.util.Log;

// TODO: Auto-generated Javadoc
/**
 * The Class RMDatabase.
 */
/**
 * @author Besitzer
 *
 */
public class RMDatabase{

	/** The m database. */
	private SQLiteDatabase mDatabase = null; 
	
	/** The m error message. */
	private String mErrorMessage = ""; 

	/** The m new version. */
	private boolean mNewVersion = false;
	
	/**
	 * Open database on sd card.
	 *
	 * @param databaseName the database name
	 * @return true, if successful
	 */
	public boolean openDatabaseOnSDCard(String databaseName){
		// Datenbank erzeugen und ffnen
		if (! openOrCreateDatabaseOnSDCard(databaseName)){
			mErrorMessage = "Fehler beim ffnen der Datenbank!";
			return false;
		}
		return true;
	}
	
	/**
	 * Open or create database on sd card.
	 * 
	 * @param dbName the db name
	 * @return true, if successful
	 */
	public boolean openOrCreateDatabaseOnSDCard(String dbName) {

		boolean success = false;
		
		try {

			// Datenbank auf SD-Karte erzeugen anschl. ffnen
			String path = "/sdcard/"+dbName;
			mDatabase = SQLiteDatabase.openOrCreateDatabase(path, null);

			// Datenbank angelegt und erfolgreich geffnet. 
			success = true;

		}
		catch (SQLiteException ex) {
			// Error
			Log.e("MyApp", "error -- " + ex.getMessage(), ex);
		}

		return success;
	}

	/**
	 * Checks if is open.
	 *
	 * @return true, if is open
	 */
	public boolean isOpen(){
		return mDatabase.isOpen();
	}

	/**
	 * Open.
	 *
	 * @return true, if successful
	 */
	public boolean open(String database){

		// Datenbank ffnen
		openDatabaseOnSDCard(database);
		
		// Wenn sich die Versionsnummer ndert, werden 
		// Tabellen erzeugt und Strukturen angepasst.
		// 0 = Es wird immer angepasst...
		setVersion(1);
		
		// Wenn nicht erfolgreich geffnet dann raus 
		if(!isOpen()){
			return false;
		}

		// Bei einer neuen Version wird hier die Datenbank angepasst.
		validateDatabase();
		
		return true;
	}

	/**
	 * Close.
	 */
	public void close(){
		if(mDatabase != null){
			mDatabase.close();
		}
	}

	/**
	 * Gets the database.
	 *
	 * @return the database
	 */
	public SQLiteDatabase getDatabase(){
		return mDatabase;
	}

	/**
	 * Gets the error message.
	 *
	 * @return the error message
	 */
	public String getErrorMessage(){
		return mErrorMessage;
	}
	
	/**
	 * Sets the version.
	 *
	 * @param version the new version
	 */
	public void setVersion(int version){
		if(version != mDatabase.getVersion() ){
			mNewVersion = true;
		}else{
			if(version == 0) mNewVersion = true;
		}
		mDatabase.setVersion(version);
	}

	/**
	 * Gets the version.
	 *
	 * @return the version
	 */
	public int getVersion(){
		return mDatabase.getVersion();
	}
	

	/**
	 * Validate database.
	 *
	 * @return true, if successful
	 */
	public boolean validateDatabase() {

		if(! mNewVersion) return true;

		Log.d("MyApp", "Datenbank Verion "+getVersion()+" wird aktualisiert...");

//		// Nurm zum testen....
//		if (execute("DROP TABLE 'DICTIONARY'")){
//			Log.d("MyApp","DICT.. gelscht....");
//		}
		
		// Erzeuge Tabellen wenn sie nich existieren
		createTableStructure();

		// Erzeugt einen Trigger fr das Feld MyDateTime. Dort wird bei jeder 
		// Neuanlage das aktuelle Datum und die aktuelle Zeit eingetragen.
		createTrigger();
		
		// Die Struktur einer laufenden Datenbank erweitern.
		// Es gehen keine Daten verloren.
		changeTableStructure();
		
		// Ein paar Datenstze hinzufgen
		addRecord();
		
		return true;
	}

		/**
	 * Creates the table structure.
	 *
	 * Geht nur, wenn die Tabelle nicht exisitert. Sonst ist
	 * changeTableStructure() fr Erweiterungen zustndig. 
	 *
	 * @return true, if successful
	 */
	private boolean createTableStructure() {
		boolean success = false;
		
		try {

			// INTEGER PRIMARY KEY AUTOINCREMENT
			// Der Primrschlssel hier mit einem Autowert kann nur beim 
			// erstellen der Tabelle angelegt werden. Nachtrglich
			// geht das nicht. Jede Tabelle hat einen INTEGER PRIMARY KEY, 
			// dieser ist als ROWID, OID oder ROWID  erreichbar. Definiert 
			// man diese Spalte selber (evtl. mit AUTOINCREMENT), dann wird 
			// diese Spalte verwendet, ansonsten wird eine intern erzeugt 
			// es ist die ID im B-Tree). 
			
			// UNIQUE
			// Doppelte Eintrge sind mit UNIQUE nicht mglich.
			// Nachtrglich siehe changeTableStructure konnte ich mit ALTER TABLE
			// solch ein Feld nicht anlegen. In diesem Fall geht es mit einem
			// selbst erzeugten Index. Siehe ebenfalls changeTableStructure.
			
			// NOT NULL
			// Es wir ein Wert erwartet. Nullwerte sind nicht erlaubt.
			
			// Datentypen!!! 
			// Die in Klammern angegebenen Begrenzungen bsp. VARCHAR(10) werden
			// in SQLite 3 nicht bercksichtigt. Hier die wichtigsten....
			// Mehr unter http://www.sqlite.org/datatype3.html

			// Tabelle1 ertsellen. Leerschritt hinter Tabelle1 ist wichtig.
			String sql = "CREATE TABLE IF NOT EXISTS DICTIONARY ";

			// Feldnamen ebenfalls anlegen. 
			// Diese werden in runden Klammern umfasst.
			sql = sql + "(";

			// Der Primrschlssel hier mit einem Autowert kann nur beim 
			// erstellen der Tabelle angelegt werden. Nachtrglich
			// geht das nicht. Es ist jedoch mglich in solch einem Falls
			// auf das Feld ROWID oder OID zurck zu greifen.
			sql = sql + "_id INTEGER PRIMARY KEY AUTOINCREMENT, ";

			// TEXT
			// Folgende Werte knnen verwendet werden: NULL, TEXT, BLOB
			
			sql = sql + "Source TEXT, ";
			sql = sql + "Target TEXT ";
			
			// Klammer zum schlieen nicht vergessen
			sql = sql + ")";
			
			// Syntax im Debugger (LogCat) ausgeben
			Log.d("MyApp","sql : "+sql);
			
			// SQL-Anweisung ausfhren
			mDatabase.execSQL(sql);
			
			// Wenn kein Fehler alles OK!
			success = true;

		}
		catch (SQLiteException ex) {
			Log.e("MyApp", "error createTableStructure -- " + ex.getMessage(), ex);
			// error means tables does not exits
		}
		return success;
	}

	/**
	 * Change table structure.
	 *
	 * @return true, if successful
	 */
	private boolean changeTableStructure() {

//		// Spalte nachtrglich hinzufgen
//		if(execute("ALTER TABLE Tabelle1 ADD COLUMN New_Column1 TEXT DEFAULT 'Ihre Eingabe'")){
//			Log.d("MyApp", "ERFOLGREICH ALTER TABLE 1");
//		}
//		
//		if(execute("ALTER TABLE Tabelle1 ADD COLUMN New_Column2 TEXT DEFAULT 'Ihre Eingabe'")){
//			Log.d("MyApp", "ERFOLGREICH ALTER TABLE 2");
//		}
//
//		// Zwei weitere Felder (Index ist notwendig) werden als einzigartig hinzugefgt. 
//		// Besonders ist, das bei der indizierung die Einzigartigkeit aus beiden 
//		// Feldern besteht.
//		execute("ALTER TABLE Tabelle1 ADD COLUMN New_Unique1");
//		execute("ALTER TABLE Tabelle1 ADD COLUMN New_Unique2");
//	
//		// Einen Index von Tabelle1 der Felder New_Unique1 + New_Unique2 erzeugen.
//		execute("CREATE UNIQUE INDEX Tabelle1_Index1 ON Tabelle1(New_Unique1, New_Unique2)");
		
		return true;
	}
		
	/**
	 * Creates the trigger.
	 *
	 * @return true, if successful
	 */
	private boolean createTrigger() {
		
		// Mit diesem Trigger ist es mglich dem Feld MyDateTime das aktuelle Datum und die
		// aktuelle Uhrzeit bei jeder Neuanlage mitzugeben. Beim erstellen der Tabelle 
		// ber DEFAULT geht das leider nicht.
		
//		String trigger = "CREATE TRIGGER insert_Tabelle1_DateTime AFTER INSERT ON Tabelle1 "+
//		" BEGIN UPDATE Tabelle1 SET MyDateTime = DATETIME('NOW') WHERE rowid = new.rowid; END; ";
//		
//		if (!execute(trigger)){
//			Log.d("MyApp","Trigger fr MyDateTime nicht erstellt");			
//		}else{
//			Log.d("MyApp","Trigger fr MyDateTime erfolgreich erstellt");			
//		}
		
		return true;
		
	}

	/**
	 * Execute.
	 *
	 * @param sql the sql
	 * @return true, if successful
	 */
	public boolean execute(String sql) {
		boolean success = false;
		try {
			mDatabase.execSQL(sql);
			success = true;
		}
		catch (SQLiteException ex) {
			// Error
			Log.e("MyApp", "error execute -- " + ex.getMessage(), ex);
		}
		return success;
	}

	/**
	 * Adds the record.
	 *
	 * @return true, if successful
	 */
	private void addRecord() {
		ContentValues values = new ContentValues();
		int count = 1;
        while (count <= 10) {
			values.put("Source", "Eintrag "+count);
			values.put("Target", "Entry "+count);
			mDatabase.insert("DICTIONARY", null, values);
            count++;
            Log.d("MyApp","Count "+count+" von 10");
        }
	}
}
